/** 
 * Copyright (c) 2010 Keang Ltd
 * 
 * This file is part of the Date Selector package.
 *
 * The Date Selector package is free software: you can redistribute it and/or 
 * modify it under the terms of the GNU General Public License as published 
 * by the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * The Date Selector package is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY 
 * or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for 
 * more details.
 * 
 * You should have received a copy of the GNU General Public License along
 * with the Date Selector package.  If not, see <http://www.gnu.org/licenses/>.
 */
package uk.co.keang.date.demo;

import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.Font;
import java.awt.GridLayout;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.util.Vector;
import javax.swing.JButton;
import javax.swing.JCheckBox;
import javax.swing.JComboBox;
import javax.swing.JLabel;
import javax.swing.JList;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.SwingConstants;
import javax.swing.border.CompoundBorder;
import javax.swing.border.EmptyBorder;
import javax.swing.border.EtchedBorder;
import uk.co.keang.date.AbstractDateSelector;
import uk.co.keang.date.AbstractDateSelector.DayButton;
import uk.co.keang.swingaddons.SelectableLineBorder;

/**
 * A test GUI for the JDateTimeSelector class
 *
 * @author A.G.Docherty
 * @date 20 Jan 2010
 *
 */
public abstract class AbstractSelectorDemo extends JPanel
{
private static final long serialVersionUID = 8070920177737844373L;
private static final int MAX_MESSAGES = 8;
protected JButton dateBtn;
protected JLabel hoverLbl;
protected AbstractDateSelector embeddedPanel;
protected boolean showWeekNumbers = true;
protected boolean showDayNames = true;
protected Font font;
private JList actionList;
private Vector<String> msgs = new Vector<String>();
protected boolean showOtherDays = true;

protected AbstractSelectorDemo()
    {
    initObject();
    initPanel();
    }

private void initPanel()
    {
    setBorder(new EmptyBorder(5,5,5,5));
    setLayout(new BorderLayout(5,5));

    JPanel top = new JPanel();
    top.setLayout(new GridLayout(0,2,10,10));
    top.setBorder(new CompoundBorder(new SelectableLineBorder(SelectableLineBorder.Side.BOTTOM, Color.BLACK),
            new EmptyBorder(5,5,10,5)));

    dateBtn = new JButton("Click to Select a Date");
    dateBtn.addActionListener(new ActionListener()
        {
        @Override
        public void actionPerformed(ActionEvent e)
            {
            showSelectorDialog();
            }
        });

    top.add(dateBtn);

    hoverLbl = new JLabel("Hover to Select a Date", SwingConstants.CENTER);
    hoverLbl.setBorder(new EtchedBorder());

    createHoverOnPopup();
    top.add(hoverLbl);

    JCheckBox chk = new JCheckBox("Show week numbers", showWeekNumbers);
    chk.addActionListener(new ActionListener()
        {
        @Override
        public void actionPerformed(ActionEvent e)
            {
            showWeekNumbers = ((JCheckBox)e.getSource()).isSelected();
            embeddedPanel.setShowWeekNums(showWeekNumbers);
            }
        });

    top.add(chk);

    chk = new JCheckBox("Show day names", showDayNames);
    chk.addActionListener(new ActionListener()
        {
        @Override
        public void actionPerformed(ActionEvent e)
            {
            showDayNames = ((JCheckBox)e.getSource()).isSelected();
            embeddedPanel.setShowDayNames(showDayNames);
            }
        });

    top.add(chk);

    chk = new JCheckBox("Show other month days", showOtherDays);
    chk.addActionListener(new ActionListener()
        {
        @Override
        public void actionPerformed(ActionEvent e)
            {
            showOtherDays = ((JCheckBox)e.getSource()).isSelected();
            embeddedPanel.setShowSurroundingDays(showOtherDays);
            }
        });

    top.add(chk);

    JPanel p = new JPanel();
    p.add(new JLabel("Font Size", SwingConstants.RIGHT));
    JComboBox fontCB = new JComboBox(new Integer[]{6,7,8,9,10,11,12,14,16,18,20});
    
    font = getFont();
    fontCB.setSelectedItem(Integer.valueOf(font.getSize()));
    fontCB.addActionListener(new ActionListener() 
        {
        @Override
        public void actionPerformed(ActionEvent e)
            {
            // set the font here
            Font f = embeddedPanel.getFont();
            font = f.deriveFont(((Integer)((JComboBox)e.getSource()).getSelectedItem()).floatValue());
            
            embeddedPanel.setFont(font);
            }
        });
    
    p.add(fontCB);
    top.add(p);
    
    add(top, BorderLayout.NORTH);

    embeddedPanel = createEmbeddedPanel();
    embeddedPanel.addDayActionListener(new ActionListener() 
        {
        @Override
        public void actionPerformed(ActionEvent e)
            {
            DayButton btn = ((DayButton)e.getSource());
            String msg = "Day Button clicked for day No. "+btn.getDay();
            displayMessage(msg);
            }
        });
    add(embeddedPanel, BorderLayout.CENTER);
    
    actionList = new JList();
    add(new JScrollPane(actionList), BorderLayout.SOUTH);
    }


protected abstract void createHoverOnPopup();

protected abstract AbstractDateSelector createEmbeddedPanel();

protected abstract void showSelectorDialog();


/**
 * Displays a message in the JList
 * 
 * @param msg
 */
protected void displayMessage(String msg)
    {
    msgs.add(msg);
    if ( msgs.size() > MAX_MESSAGES )
        msgs.removeElementAt(0);
    
    actionList.setListData(msgs);
    embeddedPanel.requestFocus();
    }

protected void initObject()
    {
    // TODO Auto-generated method stub
    
    }

}
