/**
 * Copyright (c) 2010 Keang Ltd
 * 
 * This file is part of the Date Selector package.
 *
 * The Date Selector package is free software: you can redistribute it and/or 
 * modify it under the terms of the GNU General Public License as published 
 * by the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * The Date Selector package is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY 
 * or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for 
 * more details.
 * 
 * You should have received a copy of the GNU General Public License along
 * with the Date Selector package.  If not, see <http://www.gnu.org/licenses/>.
 */
package uk.co.keang.date.demo;

import java.awt.Dimension;
import java.awt.Toolkit;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import javax.swing.JFrame;
import javax.swing.UIManager;
import javax.swing.event.ChangeEvent;
import javax.swing.event.ChangeListener;
import org.joda.time.DateTime;
import org.joda.time.MutableDateTime;
import org.joda.time.format.ISODateTimeFormat;
import uk.co.keang.date.AbstractDateSelector;
import uk.co.keang.date.DateSelector;

/**
 * A test GUI for the JDateTimeSelector class
 *
 * @author A.G.Docherty
 * @date 20 Jan 2010
 *
 */
public class DateSelectorDemo extends AbstractSelectorDemo
{
private static final long serialVersionUID = 6306653208309519528L;
protected MutableDateTime selectedDate;


/**
* Displays a modal dialog allowing the selection of a date.
* This method is provided to allow a Calendar pane to be constructed, configured
* and then displayed.
*
* @param panel - the panel
* @return the selected date or null if cancel pressed
*/
public static DateTime show(DateSelectorDemo panel)
   {
   JFrame dlg = new JFrame("Date Selection Test");
   dlg.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
   dlg.add(panel);

   dlg.pack();

   panel.embeddedPanel.requestFocus();

   Dimension screenSize = Toolkit.getDefaultToolkit().getScreenSize();
   Dimension d = dlg.getSize();
   dlg.setLocation((screenSize.width-d.width)/2, (screenSize.height-d.height)/2);

   dlg.setVisible(true);

   return panel.selectedDate.toDateTime(panel.selectedDate.getChronology());
   }


@Override
protected void initObject()
    {
    selectedDate = new MutableDateTime();
    }

@Override
protected AbstractDateSelector createEmbeddedPanel()
    {
    DateSelector panel = new DateSelector(selectedDate.toDateTime(selectedDate.getChronology()), false);
    panel.requestFocusInWindow();
    panel.addDateChangedListener(new ChangeListener()
        {
        @Override
        public void stateChanged(ChangeEvent e)
            {
            // a new date has been selected so make a note of it
            selectedDate.setMillis(((DateSelector)embeddedPanel).getSelectedDate().getMillis());
            
            String msg = "Embedded Panel: Date changed to "+ISODateTimeFormat.date().print(selectedDate);
            displayMessage(msg);
            }
        });

    return panel;
    }


@Override
protected void createHoverOnPopup()
    {
    DateSelector.setPopupOnHoverForComponent(hoverLbl, selectedDate, new ActionListener()
        {
        @Override
        public void actionPerformed(ActionEvent e)
            {
            // a new date has been selected so make a note of it and set the
            // embedded panel to the new date
            selectedDate.setMillis(((DateSelector)e.getSource()).getSelectedDate().getMillis());
            
            String msg = "Hover Panel: Date changed to "+ISODateTimeFormat.date().print(selectedDate);
            displayMessage(msg);

            ((DateSelector)embeddedPanel).setSelectedDate(selectedDate.toDateTime(selectedDate.getChronology()));
            }
        });
    }


@Override
protected void showSelectorDialog()
    {
    DateSelector dts = new DateSelector(selectedDate.toDateTime(selectedDate.getChronology()), true);
    dts.setShowWeekNums(showWeekNumbers);
    dts.setShowDayNames(showDayNames);
    dts.setShowSurroundingDays(showOtherDays);
    dts.setFont(font);
    
    DateTime d = DateSelector.show(dateBtn, "Select Date", dts);

    if ( d != null )
        {
        // a new date has been selected so make a note of it and set the embedded panel
        // to the new date
        selectedDate.setDate(d.getMillis());

        String msg = "Dialog Panel: Date changed to "+ISODateTimeFormat.date().print(selectedDate);
        displayMessage(msg);

        ((DateSelector)embeddedPanel).setSelectedDate(d);
        }
    }

/**
 * Run the calendar
 *
 * @param args
 */
public static void main(String[] args)
    {
    try
        {
        UIManager.setLookAndFeel(UIManager.getSystemLookAndFeelClassName());
        }
    catch ( Exception e )
        {
        // ignore this as it will still display - it just won't be the system look and feel
        }

    DateSelectorDemo cp = new DateSelectorDemo();
    show(cp);
    }
}
